#include "base/exception.hpp"
#include "base/objectlock.hpp"
#include "base/utility.hpp"
#include "base/convert.hpp"
#include "base/dependencygraph.hpp"
#include "base/logger.hpp"
#include "base/function.hpp"
#include "base/configtype.hpp"
#ifdef _MSC_VER
#pragma warning( push )
#pragma warning( disable : 4244 )
#pragma warning( disable : 4800 )
#endif /* _MSC_VER */

namespace icinga
{

TypeImpl<OpenTsdbWriter>::TypeImpl()
{ }

TypeImpl<OpenTsdbWriter>::~TypeImpl()
{ }

String TypeImpl<OpenTsdbWriter>::GetName() const
{
	return "OpenTsdbWriter";
}

int TypeImpl<OpenTsdbWriter>::GetAttributes() const
{
	return 0;
}

Type::Ptr TypeImpl<OpenTsdbWriter>::GetBaseType() const
{
	return ConfigObject::TypeInstance;
}

int TypeImpl<OpenTsdbWriter>::GetFieldId(const String& name) const
{
	int offset = ConfigObject::TypeInstance->GetFieldCount();

	switch (static_cast<int>(Utility::SDBM(name, 1))) {
		case 104:
			if (name == "host")
				return offset + 0;

			break;
		case 112:
			if (name == "port")
				return offset + 1;

			break;
	}

	return ConfigObject::TypeInstance->GetFieldId(name);
}

Field TypeImpl<OpenTsdbWriter>::GetFieldInfo(int id) const
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount();
	if (real_id < 0) { return ConfigObject::TypeInstance->GetFieldInfo(id); }
	switch (real_id) {
		case 0:
			return {0, "String", "host", "host", nullptr, 2, 0};
		case 1:
			return {1, "String", "port", "port", nullptr, 2, 0};
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

int TypeImpl<OpenTsdbWriter>::GetFieldCount() const
{
	return 2 + ConfigObject::TypeInstance->GetFieldCount();
}

ObjectFactory TypeImpl<OpenTsdbWriter>::GetFactory() const
{
	return TypeHelper<OpenTsdbWriter, false>::GetFactory();
}

std::vector<String> TypeImpl<OpenTsdbWriter>::GetLoadDependencies() const
{
	std::vector<String> deps;
	return deps;
}

int TypeImpl<OpenTsdbWriter>::GetActivationPriority() const
{
	return 100;
}

void TypeImpl<OpenTsdbWriter>::RegisterAttributeHandler(int fieldId, const Type::AttributeHandler& callback)
{
	int real_id = fieldId - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { ConfigObject::TypeInstance->RegisterAttributeHandler(fieldId, callback); return; }
	switch (real_id) {
		case 0:
			ObjectImpl<OpenTsdbWriter>::OnHostChanged.connect(callback);
			break;
		case 1:
			ObjectImpl<OpenTsdbWriter>::OnPortChanged.connect(callback);
			break;
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

void ObjectImpl<OpenTsdbWriter>::Validate(int types, const ValidationUtils& utils)
{
	ConfigObject::Validate(types, utils);

	if (2 & types)
		ValidateHost(Lazy<String>([this]() { return GetHost(); }), utils);
	if (2 & types)
		ValidatePort(Lazy<String>([this]() { return GetPort(); }), utils);
}

void ObjectImpl<OpenTsdbWriter>::SimpleValidateHost(const Lazy<String>& value, const ValidationUtils& utils)
{
}

void ObjectImpl<OpenTsdbWriter>::SimpleValidatePort(const Lazy<String>& value, const ValidationUtils& utils)
{
}

ObjectImpl<OpenTsdbWriter>::ObjectImpl()
{
	SetHost(GetDefaultHost(), true);
	SetPort(GetDefaultPort(), true);
}

ObjectImpl<OpenTsdbWriter>::~ObjectImpl()
{ }

void ObjectImpl<OpenTsdbWriter>::SetField(int id, const Value& value, bool suppress_events, const Value& cookie)
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { ConfigObject::SetField(id, value, suppress_events, cookie); return; }
	switch (real_id) {
		case 0:
			SetHost(value, suppress_events, cookie);
			break;
		case 1:
			SetPort(value, suppress_events, cookie);
			break;
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

Value ObjectImpl<OpenTsdbWriter>::GetField(int id) const
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { return ConfigObject::GetField(id); }
	switch (real_id) {
		case 0:
			return GetHost();
		case 1:
			return GetPort();
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

void ObjectImpl<OpenTsdbWriter>::ValidateField(int id, const Lazy<Value>& lvalue, const ValidationUtils& utils)
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { ConfigObject::ValidateField(id, lvalue, utils); return; }
	switch (real_id) {
		case 0:
			ValidateHost(lvalue, utils);
			break;
		case 1:
			ValidatePort(lvalue, utils);
			break;
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

void ObjectImpl<OpenTsdbWriter>::NotifyField(int id, const Value& cookie)
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { ConfigObject::NotifyField(id, cookie); return; }
	switch (real_id) {
		case 0:
			NotifyHost(cookie);
			break;
		case 1:
			NotifyPort(cookie);
			break;
		default:
			throw std::runtime_error("Invalid field ID.");
	}
}

Object::Ptr ObjectImpl<OpenTsdbWriter>::NavigateField(int id) const
{
	int real_id = id - ConfigObject::TypeInstance->GetFieldCount(); 
	if (real_id < 0) { return ConfigObject::NavigateField(id); }
	throw std::runtime_error("Invalid field ID.");
}

String ObjectImpl<OpenTsdbWriter>::GetHost() const
{
	return m_Host;
}

String ObjectImpl<OpenTsdbWriter>::GetPort() const
{
	return m_Port;
}

void ObjectImpl<OpenTsdbWriter>::SetHost(const String& value, bool suppress_events, const Value& cookie)
{
	m_Host = value;
	if (!suppress_events)
		NotifyHost(cookie);
}

void ObjectImpl<OpenTsdbWriter>::SetPort(const String& value, bool suppress_events, const Value& cookie)
{
	m_Port = value;
	if (!suppress_events)
		NotifyPort(cookie);
}

void ObjectImpl<OpenTsdbWriter>::NotifyHost(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnHostChanged(static_cast<OpenTsdbWriter *>(this), cookie);
}

void ObjectImpl<OpenTsdbWriter>::NotifyPort(const Value& cookie)
{
	auto *dobj = dynamic_cast<ConfigObject *>(this);
	if (!dobj || dobj->IsActive())
		OnPortChanged(static_cast<OpenTsdbWriter *>(this), cookie);
}

String ObjectImpl<OpenTsdbWriter>::GetDefaultHost() const
{
	 return "127.0.0.1"; 
}

String ObjectImpl<OpenTsdbWriter>::GetDefaultPort() const
{
	 return "4242"; 
}


boost::signals2::signal<void (const intrusive_ptr<OpenTsdbWriter>&, const Value&)> ObjectImpl<OpenTsdbWriter>::OnHostChanged;


boost::signals2::signal<void (const intrusive_ptr<OpenTsdbWriter>&, const Value&)> ObjectImpl<OpenTsdbWriter>::OnPortChanged;

void ObjectImpl<OpenTsdbWriter>::ValidateHost(const Lazy<String>& lvalue, const ValidationUtils& utils)
{
	SimpleValidateHost(lvalue, utils);
}

void ObjectImpl<OpenTsdbWriter>::ValidatePort(const Lazy<String>& lvalue, const ValidationUtils& utils)
{
	SimpleValidatePort(lvalue, utils);
}

}
#ifdef _MSC_VER
#pragma warning ( pop )
#endif /* _MSC_VER */
